package cz.muni.fi.pa165;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.security.oauth2.client.OAuth2AuthorizedClient;
import org.springframework.security.oauth2.client.annotation.RegisteredOAuth2AuthorizedClient;
import org.springframework.security.oauth2.core.OAuth2AccessToken;
import org.springframework.security.oauth2.core.oidc.user.OidcUser;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;

@Controller
public class MainController {

    private static final Logger log = LoggerFactory.getLogger(MainController.class);

    /**
     * Handles requests to the "/" endpoint and returns the index view.
     *
     * @param model the model used to pass data to the view
     * @param user  the authenticated OidcUser, or null if the user is anonymous
     * @return the name of the index view
     */
    @GetMapping("/")
    public String index(Model model, @AuthenticationPrincipal OidcUser user) {
        log.debug("********************************************************");
        log.debug("* index() called                                       *");
        log.debug("********************************************************");
        log.debug("user {}", user == null ? "is anonymous" : user.getSubject());

        model.addAttribute("user", user);

        if (user != null) {
            model.addAttribute("issuerName",
                    "https://oidc.muni.cz/oidc/".equals(user.getIssuer().toString()) ? "MUNI" : "Google");
        }

        return "index";
    }

    /**
     * Handles requests to the "/token" endpoint and returns the token view.
     *
     * @param model        the model used to pass data to the view
     * @param oauth2Client the registered OAuth2AuthorizedClient used to obtain the access token
     * @return the name of the token view
     */
    @GetMapping("/token")
    public String getMeToken(Model model, @RegisteredOAuth2AuthorizedClient OAuth2AuthorizedClient oauth2Client) {
        log.info("********************************************************");
        log.info("* token() called                                  *");
        log.info("********************************************************");

        OAuth2AccessToken accessToken = oauth2Client.getAccessToken();
        model.addAttribute("accessTokenValue", accessToken.getTokenValue());
        return "token";
    }
}
