package cz.muni.fi.pa165.movie;


import cz.muni.fi.pa165.common.BaseMapper;
import cz.muni.fi.pa165.image.Image;
import cz.muni.fi.pa165.image.ImageMapper;
import org.mapstruct.CollectionMappingStrategy;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Mapper(uses = ImageMapper.class,
        collectionMappingStrategy = CollectionMappingStrategy.SETTER_PREFERRED)
public interface MovieMapper extends BaseMapper<Movie, MovieDto> {

    /**
     * Updates the given {@code movie} entity with the values from the given {@code movieDto}.
     * Images are also updated, and new images are created if necessary.
     *
     * @param movieDto the DTO instance to update the entity from
     * @param movie the entity to update
     */
    default void updateFromDto(MovieDto movieDto, @MappingTarget Movie movie) {
        ImageMapper imageMapper = Mappers.getMapper(ImageMapper.class);
        Map<Long, Image> existingImages = movie.getImages().stream()
                .collect(Collectors.toMap(Image::getId, Function.identity()));

        movie.setTitle(movieDto.getTitle());
        movie.setDescription(movieDto.getDescription());
        movie.setDirector(movieDto.getDirector());
        movie.setGenres(movieDto.getGenres());
        movie.setActors(movieDto.getActors());

        List<Image> updatedImages = movieDto.getImages().stream()
                .map(imageDto -> {
                    if (imageDto.getId() != null && existingImages.containsKey(imageDto.getId())) {
                        Image existingImage = existingImages.get(imageDto.getId());
                        imageMapper.updateFromDto(imageDto, existingImage);
                        return existingImage;
                    } else {
                        Image newImage = imageMapper.fromDto(imageDto);
                        newImage.setMovie(movie);
                        return newImage;
                    }
                }).collect(Collectors.toList());
        movie.setImages(updatedImages);
    }
}
