package cz.muni.fi.pa165.seed;

import cz.muni.fi.pa165.CoreApplication;
import cz.muni.fi.pa165.image.Image;
import cz.muni.fi.pa165.movie.GenreEnum;
import cz.muni.fi.pa165.movie.Movie;
import cz.muni.fi.pa165.movie.MovieRepository;
import cz.muni.fi.pa165.rating.Rating;
import cz.muni.fi.pa165.rating.RatingCategoryEnum;
import cz.muni.fi.pa165.rating.RatingRepository;
import cz.muni.fi.pa165.user.Role;
import cz.muni.fi.pa165.user.User;
import cz.muni.fi.pa165.user.UserRepository;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Random;

@Controller
@RestController
@RequestMapping(path = "/seed")
public class SeedController {

    @Autowired
    private UserRepository userRepository;
    @Autowired
    private MovieRepository movieRepository;
    @Autowired
    private RatingRepository ratingRepository;

    @Operation(summary = "Seed database",
            security = @SecurityRequirement(name = CoreApplication.SECURITY_SCHEME_NAME, scopes = {"test_write"})
    )
    @GetMapping
    public void seedDatabase() {
        List<User> users = createUsers();
        List<Movie> movies = createMovies();
        createRatings(movies, users);
    }

    private List<User> createUsers() {
        List<User> users = Arrays.asList(
                User.builder().username("john").password("john123").firstname("John").lastname("Doe").role(Role.USER).build(),
                User.builder().username("sarah").password("sarah123").firstname("Sarah").lastname("Smith").role(Role.USER).build(),
                User.builder().username("michael").password("michael123").firstname("Michael").lastname("Johnson").role(Role.USER).build(),
                User.builder().username("emma").password("emma123").firstname("Emma").lastname("Davis").role(Role.USER).build(),
                User.builder().username("david").password("david123").firstname("David").lastname("Martinez").role(Role.ADMIN).build()
        );
        userRepository.saveAll(users);
        return users;
    }


    private List<Movie> createMovies() {
        List<Movie> movies = Arrays.asList(
                Movie.builder()
                        .title("Inception")
                        .description("A thief who steals corporate secrets through the use of dream-sharing technology is given the inverse task of planting an idea into the mind of a CEO.")
                        .director("Christopher Nolan")
                        .actors(Arrays.asList("Leonardo DiCaprio", "Ellen Page"))
                        .genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.SCIFI))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of Inception").stringData("Image data of Inception").build()))
                        .build(),
                Movie.builder()
                        .title("Interstellar")
                        .description("A team of explorers travel through a wormhole in space in an attempt to ensure humanity's survival.")
                        .director("Christopher Nolan")
                        .actors(Arrays.asList("Matthew McConaughey", "Anne Hathaway"))
                        .genres(Arrays.asList(GenreEnum.ADVENTURE, GenreEnum.DRAMA))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of Interstellar").stringData("Image data of Interstellar").build()))
                        .build(),
                Movie.builder()
                        .title("The Godfather")
                        .description("The aging patriarch of an organized crime dynasty transfers control of his empire to his reluctant son.")
                        .director("Francis Ford Coppola")
                        .actors(Arrays.asList("Marlon Brando", "Al Pacino"))
                        .genres(Arrays.asList(GenreEnum.CRIME, GenreEnum.DRAMA))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of The Godfather").stringData("Image data of The Godfather").build()))
                        .build(),
                Movie.builder()
                        .title("The Dark Knight")
                        .description("When the menace known as the Joker wreaks havoc and chaos on the people of Gotham, Batman must accept one of the greatest psychological and physical tests of his ability to fight injustice.")
                        .director("Christopher Nolan")
                        .actors(Arrays.asList("Christian Bale", "Heath Ledger"))
                        .genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.CRIME, GenreEnum.DRAMA))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of The Dark Knight").stringData("Image data of The Dark Knight").build()))
                        .build(),
                Movie.builder()
                        .title("The Shawshank Redemption")
                        .description("Two imprisoned men bond over a number of years, finding solace and eventual redemption through acts of common decency.")
                        .director("Frank Darabont")
                        .actors(Arrays.asList("Tim Robbins", "Morgan Freeman"))
                        .genres(List.of(GenreEnum.DRAMA))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of The Shawshank Redemption").stringData("Image data of The Shawshank Redemption").build()))
                        .build(),
                Movie.builder()
                        .title("Pulp Fiction")
                        .description("The lives of two mob hitmen, a boxer, a gangster's wife, and a pair of diner bandits intertwine in four tales of violence and redemption.")
                        .director("Quentin Tarantino")
                        .actors(Arrays.asList("John Travolta", "Uma Thurman"))
                        .genres(Arrays.asList(GenreEnum.CRIME, GenreEnum.DRAMA))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of Pulp Fiction").stringData("Image data of Pulp Fiction").build()))
                        .build(),
                Movie.builder()
                        .title("Fight Club")
                        .description("An insomniac office worker and a devil-may-care soap maker form an underground fight club that evolves into something much, much more.")
                        .director("David Fincher")
                        .actors(Arrays.asList("Brad Pitt", "Edward Norton"))
                        .genres(List.of(GenreEnum.DRAMA))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of Fight Club").stringData("Image data of Fight Club").build()))
                        .build(),
                Movie.builder()
                        .title("The Matrix")
                        .description("A computer hacker learns from mysterious rebels about the true nature of his reality and his role in the war against its controllers.")
                        .director("Lana Wachowski, Lilly Wachowski")
                        .actors(Arrays.asList("Keanu Reeves", "Laurence Fishburne"))
                        .genres(Arrays.asList(GenreEnum.ACTION, GenreEnum.SCIFI))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of The Matrix").stringData("Image data of The Matrix").build()))
                        .build(),
                Movie.builder()
                        .title("Forrest Gump")
                        .description("The presidencies of Kennedy and Johnson, the events of Vietnam, Watergate, and other historical events unfold through the perspective of an Alabama man with an IQ of 75, whose only desire is to be reunited with his childhood sweetheart.")
                        .director("Robert Zemeckis")
                        .actors(Arrays.asList("Tom Hanks", "Robin Wright"))
                        .genres(Arrays.asList(GenreEnum.DRAMA, GenreEnum.ROMANCE))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of Forrest Gump").stringData("Image data of Forrest Gump").build()))
                        .build(),
                Movie.builder()
                        .title("The Lord of the Rings: The Return of the King")
                        .description("Gandalf and Aragorn lead the World of Men against Sauron's army to draw his gaze from Frodo and Sam as they approach Mount Doom with the One Ring.")
                        .director("Peter Jackson")
                        .actors(Arrays.asList("Elijah Wood", "Viggo Mortensen"))
                        .genres(Arrays.asList(GenreEnum.ADVENTURE, GenreEnum.DRAMA, GenreEnum.FANTASY))
                        .images(Collections.singletonList(
                                Image.builder().caption("Image of The Lord of the Rings: The Return of the King").stringData("Image data of The Lord of the Rings: The Return of the King").build()))
                        .build()
                );
        movieRepository.saveAll(movies);
        return movies;
    }

    private void createRatings(List<Movie> movies, List<User> users) {
        Random random = new Random();
        List<Rating> ratings = Arrays.asList(
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(0).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(1).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(2).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(3).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(4).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(5).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(6).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(7).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(8).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(9).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(0).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(1).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(2).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(3).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(4).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(5).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(6).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(7).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(8).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(9).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),

                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(9).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(8).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(7).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(6).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(5).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(4).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(3).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(2).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(1).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(0).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(9).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(8).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(7).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(6).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(5).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(4).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(3).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(2).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(1).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(0).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),

                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(0).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(1).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(2).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(3).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(4).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(5).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(6).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(7).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(8).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(9).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(0).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(1).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(2).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(3).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(4).getId()).category(RatingCategoryEnum.PLOT).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(5).getId()).category(RatingCategoryEnum.CAST).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(6).getId()).category(RatingCategoryEnum.FUN).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(2).getId()).movieId(movies.get(7).getId()).category(RatingCategoryEnum.SPECIAL_EFFECTS).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(0).getId()).movieId(movies.get(8).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build(),
                Rating.builder().userId(users.get(1).getId()).movieId(movies.get(9).getId()).category(RatingCategoryEnum.OVERALL).rating(random.nextInt(5) + 1).build()
        );
        ratingRepository.saveAll(ratings);
    }
}
