import requests
import json

users_url = "http://localhost:8090/users"


def setup_users(auth_header):
    print("------ setting up users -------")
    
    create_user("John", "Doe", "johndoe", "123456", "USER", auth_header)
    create_user("Jane", "Deer", "janedeer", "123456", "USER", auth_header)
    create_user("Stephen", "Jones", "sjones", "123456", "USER", auth_header)
    
    user1 = get_user_by_username("johndoe", auth_header)
    user2 = get_user_by_username("janedeer", auth_header)
    user3 = get_user_by_username("sjones", auth_header)
    
    return [user1, user2, user3]


def create_user(firstname, lastname, username, password, role, auth_header):
    json_data = {"firstname": firstname,
                 "lastname": lastname,
                 "username": username,
                 "password": password,
                 "role": role
                 }
    response = requests.post(users_url, json=json_data, headers=auth_header)
    if response.status_code == 200:
        print(f"User {username} created.")
    else:
        print(f"Error while creating user {username}.")


def get_user_by_username(username, auth_header):
    url = f"{users_url}/{username}"
    response = requests.get(url, headers=auth_header)
    if response.status_code == 200:
        print(f"Retrieved user {username}.")
        return response.json()
    else:
        print(f"Error while retrieving user {username}.")
        return None


def delete_all_users(auth_header):
    response = requests.delete(users_url, headers=auth_header)
    if response.status_code == 200:
        print("Deleted all users.")
    else:
        print("Error while deleting all users")
